#!/usr/bin/python

# ACWP location updater by Peter D'Hoye
# based on Google Latitude updater by Anssi Kolehmainen and MasterZap
#
# Copyright (c) 2011 Anssi Kolehmainen <anssi@aketzu.net>
#
# Some modifications by MasterZap (twitter.com/MasterZap)
#  - Changed to true GPS always
#  - Discard 'nan' accuracy inputs
#  - Only allow good accuracy points
#  - INSTEAD of being called by e.g. Alarmed (like my old updater
#    script), this script will sleep the GPS itself, wait for a 
#    certain time, and then 
#  - Warning: Requires fixed install location for latitude.dat at 
#    /home/user/latitude/latitude.dat so you need to create the
#    directory /home/user/latitude or it'll break
#
# (Released under MIT Licence)
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.

# Requires python-location package

#
#  VARIABLES THAT DEFINE THE ACCURACY OF STUFF
#

EPS               = 0.001  # Points within this many degrees are considered "same" and not sent
UPDATE_AT_MOST    = 1      # NEVER update more than this (minutes) even when moving
UPDATE_AT_LEAST   = 60     # NEVER update LESS than this (minutes) even when still (to avoid "stale points" in Latitude)
GPS_INTERVAL      = 60      # How often to "awaken" the GPS (minutes)



#import httplib2
import pickle

import location
import gobject
import time
import socket
import os
import sys
import getopt
import urllib2


class LocationData:
	lat=0; lng=0; alt=0; acc=0; alt=0; altacc=0; head=0; speed=0;

	prevlat=0
	prevlng=0
	prevalt=0
	prevacc=0

	prevupd=0
	prevacc=9999

verbose=1
acclimit=150
curloc = LocationData()

fix_tries = 0

def update_loc(mode, lat, lng, acc, alt, altacc, head, speed, data):
	global verbose
        global acclimit
	global fix_tries

	if verbose:
		print "%s: loc data %f,%f (acc %f) alt %f (%f), head %f, speed %f, Mode %d, Try %d" % (time.strftime('%Y-%m-%d %H:%M:%S'),lat,lng,acc, alt, altacc, head, speed, mode, fix_tries)

	# Ignore country-size measurements
	if mode < 1:
		print "--- mode 0 ignored!"
		return

	# Skip the NaN's in accuracy
	#if acc != acc:
	#	return

    # I don't care about data of low accuracy
	#if acc > acclimit:
	#	return

	if altacc > 32000:
		altacc=0

#	fix_tries += 1

	# Try at least three times to get a "type 3 fix"
	#if mode < 3 and fix_tries < 3:
	#	return

#	fix_tries = 0



	curloc.lat=lat
	curloc.lng=lng
	curloc.acc=acc
	curloc.alt=alt
	curloc.altacc=altacc
	curloc.head=head
	curloc.speed=speed

#	if speed < 10 or speed != speed:
#		data.stop()
#		print "Sleeping GPS"

	return

def on_error(control, error, data):
    print "location error: %d..." % error
    #data.quit()

def on_changed(device, data):
	print "Location updated"
	if device.fix:
		if device.fix[1] & location.GPS_DEVICE_LATLONG_SET:
			update_loc(device.fix[0], device.fix[4], device.fix[5], device.fix[6]/1000, device.fix[7], device.fix[8], device.fix[9], device.fix[11], data)
		else:
			print "no good fix"
	else:
		print "no fix"		
	#lat, lon, eph,  altitude, epv, track, speed

def restart_gpsd(control):
	print "Starting GPS..."
	control.start()
	return True

def start_location(data):
    data.start()
    return False

def usage():
	print "Location check using ACWP"
	print " -h  This help message"

def init():
	socket.setdefaulttimeout(10)

	loop = gobject.MainLoop()
	control = location.GPSDControl.get_default()
	device = location.GPSDevice()
	control.set_properties(preferred_method=location.METHOD_ACWP, preferred_interval=location.INTERVAL_120S)

	control.connect("error-verbose", on_error, loop)
	device.connect("changed", on_changed, control)
	# control.connect("gpsd-stopped", on_stop, loop)

#	gobject.idle_add(start_location, control)
#	gobject.timeout_add(GPS_INTERVAL * 60000, restart_gpsd, control)
	control.start()
	loop.run()

def daemonize():
	pid = os.fork()
	if (pid == 0):
		os.setsid()
		pid = os.fork()
		if (pid == 0):
			os.umask(0)
		else:
			os._exit(0)
	else:
		os._exit(0)

	os.close(0);
	os.close(1);
	os.close(2);


def main(argv):
	global verbose, service
	try:
		opts, args = getopt.getopt(argv, "h", ["help" ])
	except getopt.GetoptError:
		usage()
		sys.exit(2)


	for opt,arg in opts:
		if opt in ("-h", "--help"):
			usage()
			sys.exit(0)

	init()

main(sys.argv[1:])
